import { BC_CAMERA, BC_CHUNKS_SETTINGS, BC_CURRENT_SCENE, BC_TERRAIN_SETTINGS } from "../../GlobalVariables/GlobalVariables";
import { fillWorldGenChunk, getWorldChunkById, setWorldChunkById, worldChunkExists } from "../WorldGen";
import { WorldChunk } from "../WorldChunk/WorldGenChunk";

export class WorldChunksVisibilityUpdater {
    static _chunksFillingQueue = [];
    static _currentIndex = -1;
    /**
     * @type Map<String, WorldChunk>
     */
    static _visibleChunks = new Map();
    static enableAutoWorldChunksGeneration = false;

    static _chunksVisibilityRange = {
        min: -2,
        max: 3
    };

    // static {
    //     this.visibleChunks = new Map();
    // };

    static addChunkToFillingQueue(chunk, x, y) {
        this._chunksFillingQueue.push({ chunk, x, y });
        this._currentIndex++;
    }

    /**
     * Ticker function
     */
    static handleWorldChunkFilling() {
        // console.log(tick.deltaMS);
        if (this._currentIndex < 0) return;
        fillWorldGenChunk(
            this._chunksFillingQueue[this._currentIndex].chunk,
            this._chunksFillingQueue[this._currentIndex].x,
            this._chunksFillingQueue[this._currentIndex].y
        );
        this._chunksFillingQueue.pop();
        this._currentIndex--;
    }


    /**
     * Ticker function
     */
    static handleWorldChunksVisibility() {
        let w = BC_CHUNKS_SETTINGS.width * BC_TERRAIN_SETTINGS.tileSize * BC_TERRAIN_SETTINGS.scale;
        let h = BC_CHUNKS_SETTINGS.height * BC_TERRAIN_SETTINGS.tileSize * BC_TERRAIN_SETTINGS.scale;
        let cx = Math.floor(BC_CAMERA.position.x / w);
        let cy = Math.floor(BC_CAMERA.position.y / h);
        let chunkId = cx + "_" + cy;
        // console.log(this._visibleChunks);
        if ((this._visibleChunks.has(chunkId) && !this._visibleChunks.get(chunkId).centralChunk) || !this._visibleChunks.has(chunkId)) {
            for (const visChunk of this._visibleChunks) {
                visChunk[1].drawObject.visible = false;
            }
            this._visibleChunks.clear();
            for (let i = this._chunksVisibilityRange.min; i < this._chunksVisibilityRange.max; i++) {
                for (let j = this._chunksVisibilityRange.min; j < this._chunksVisibilityRange.max; j++) {
                    let t_chunkId = (cx + i) + "_" + (cy + j);
                    if (worldChunkExists(t_chunkId)) {
                        let wChunk = getWorldChunkById(t_chunkId);
                        wChunk.centralChunk = (i === 0 && j === 0);
                        this._visibleChunks.set(t_chunkId, wChunk);
                        wChunk.drawObject.visible = true;
                    } else if (this.enableAutoWorldChunksGeneration) {
                        // console.log(t_chunkId);
                        let newChunk = new WorldChunk(false);
                        BC_CURRENT_SCENE.addObjectToSceneWithInitialization(newChunk);
                        newChunk.drawObject.position.set(w * (cx + i), h * (cy + j));
                        this.addChunkToFillingQueue(newChunk, cx + i, cy + j);
                        this._visibleChunks.set(t_chunkId, newChunk);
                        newChunk.drawObject.visible = true;
                        newChunk.centralChunk = (i === 0 && j === 0);
                        setWorldChunkById(newChunk, t_chunkId);
                    }
                }
            }
        }
    }
}
